/*
 * TestClass.java
 * Created on Apr 17, 2013
 *
 * Copyright 2013 Lithium Technologies, Inc. 
 * Emeryville, California, U.S.A.  All Rights Reserved.
 *
 * This software is the  confidential and proprietary information
 * of  Lithium  Technologies,  Inc.  ("Confidential Information")
 * You shall not disclose such Confidential Information and shall 
 * use  it  only in  accordance  with  the terms of  the  license 
 * agreement you entered into with Lithium.
 */

package lithium.classloadertest;

import lithium.classloadertest.TestClass.FinalChild;
import lithium.classloadertest.TestClass.FinalClass;
import lithium.classloadertest.TestClass.FinalClass2;
import lithium.classloadertest.TestClass.Parent;
import lithium.classloadertest.annotation.ProxyableClasses;

import org.easymock.EasyMock;
import org.easymock.IMocksControl;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(UnfinalizingTestRunner.class)
@ProxyableClasses({FinalClass.class, FinalClass2.class, Parent.class, FinalChild.class}) 
public class TestClass {
	
	@BeforeClass
	public static void beforeClass() {
		if (Thread.currentThread().getContextClassLoader() != TestClass.class.getClassLoader()) {
			throw new RuntimeException("@BeforeClass: Classloader inconsistent");
		}
	}

	@AfterClass
	public static void afterClass() {
		if (Thread.currentThread().getContextClassLoader() != TestClass.class.getClassLoader()) {
			throw new RuntimeException("@AfterClass: Classloader inconsistent");
		}
	}

	@Before
	public void before() {
		if (Thread.currentThread().getContextClassLoader() != TestClass.class.getClassLoader()) {
			throw new RuntimeException("@Before: Classloader inconsistent");
		}
	}

	@After
	public void after() {
		if (Thread.currentThread().getContextClassLoader() != TestClass.class.getClassLoader()) {
			throw new RuntimeException("@After: Classloader inconsistent");
		}
	}

	@Test
	public void testThis() {
		if (Thread.currentThread().getContextClassLoader() != TestClass.class.getClassLoader()) {
			throw new RuntimeException("Classloader inconsistent");
		}
	}
	
	@Test
	public void testAllMethodsPublic() throws Exception {
		TestClass.FinalClass.class.getDeclaredMethod("hiddenMethod").invoke(new FinalClass());
	}
	
	@Test
	public void testMockingFinal() {
		IMocksControl control = EasyMock.createStrictControl();
		TestClass.FinalClass f = control.createMock(TestClass.FinalClass.class);
		EasyMock.expect(f.cantOverrideThis()).andReturn("yes we can");
		control.replay();
		Assert.assertEquals("yes we can", f.cantOverrideThis());	
	}

	@Test
	public void testMockingParentFinal() {
		IMocksControl control = EasyMock.createStrictControl();
		TestClass.FinalChild f = control.createMock(TestClass.FinalChild.class);
		EasyMock.expect(f.parentFinalMethod()).andReturn("yes we can touch parent");
		control.replay();
		Assert.assertEquals("yes we can touch parent", f.parentFinalMethod());	
	}

	public final class FinalClass {
		public void myMethod() {
			
		}
		
		public final String cantOverrideThis() {
			return "can't touch this";
		}
		
		@SuppressWarnings("unused")
		private void hiddenMethod() {
			
		}
	}

	public final class FinalClass2 {
		public void myMethod2() {
			
		}
	}
	
	public final class FinalChild extends Parent {
	}
	
	private static final class PrivateParentImpl extends Parent {
		
		@Override
		public void overrideableMethod() {
		}
	}

	public static class Parent{
		public final String parentFinalMethod() {
			return "can't touch parent final method";
		}
		
		public void overrideableMethod() {
		}
		
		public static Parent getAHiddenImplementationOfParent() {
			return new PrivateParentImpl();
		}
	}
}

