/*
 * TestClass.java
 * Created on Apr 17, 2013
 *
 * Copyright 2013 Lithium Technologies, Inc. 
 * Emeryville, California, U.S.A.  All Rights Reserved.
 *
 * This software is the  confidential and proprietary information
 * of  Lithium  Technologies,  Inc.  ("Confidential Information")
 * You shall not disclose such Confidential Information and shall 
 * use  it  only in  accordance  with  the terms of  the  license 
 * agreement you entered into with Lithium.
 */

package lithium.classloadertest;

import java.lang.reflect.Modifier;

import lithium.classloadertest.TestClass.FinalClass;
import lithium.classloadertest.TestClass.Parent;
import lithium.classloadertest.annotation.ProxyableClasses;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(UnfinalizingTestRunner.class)
@ProxyableClasses(value={FinalClass.class}, classNames= {"lithium.classloadertest.TestClass$PrivateParentImpl"}) 
public class TestClassForNonVisibleClasses {
	
	@BeforeClass
	public static void beforeClass() {
		if (Thread.currentThread().getContextClassLoader() != TestClassForNonVisibleClasses.class.getClassLoader()) {
			throw new RuntimeException("@BeforeClass: Classloader inconsistent");
		}
	}

	@AfterClass
	public static void afterClass() {
		if (Thread.currentThread().getContextClassLoader() != TestClassForNonVisibleClasses.class.getClassLoader()) {
			throw new RuntimeException("@AfterClass: Classloader inconsistent");
		}
	}

	@Before
	public void before() {
		if (Thread.currentThread().getContextClassLoader() != TestClassForNonVisibleClasses.class.getClassLoader()) {
			throw new RuntimeException("@Before: Classloader inconsistent");
		}
	}

	@After
	public void after() {
		if (Thread.currentThread().getContextClassLoader() != TestClassForNonVisibleClasses.class.getClassLoader()) {
			throw new RuntimeException("@After: Classloader inconsistent");
		}
	}

	@Test
	public void testProxyingNonVisibleClass() {
		Parent parent = Parent.getAHiddenImplementationOfParent();
		parent.overrideableMethod();
		Assert.assertEquals("Checking class name", parent.getClass().getSimpleName(), "PrivateParentImpl");
		Assert.assertTrue("Private Class should have been made public", Modifier.isPublic(parent.getClass().getModifiers()));		
	}
	
}

