/*
 * MultinodeFunctionalTestRunnerTest.java
 * Created on Apr 17, 2013
 *
 * Copyright 2013 Lithium Technologies, Inc. 
 * Emeryville, California, U.S.A.  All Rights Reserved.
 *
 * This software is the  confidential and proprietary information
 * of  Lithium  Technologies,  Inc.  ("Confidential Information")
 * You shall not disclose such Confidential Information and shall 
 * use  it  only in  accordance  with  the terms of  the  license 
 * agreement you entered into with Lithium.
 */

package lithium.classloadertest;

import java.lang.reflect.Modifier;

import lithium.classloadertest.TestClass.FinalClass;
import lithium.classloadertest.TestClass.FinalClass2;
import lithium.classloadertest.TestClass.Parent;

import org.easymock.EasyMock;
import org.easymock.IMocksControl;
import org.junit.Assert;
import org.junit.Test;
import org.junit.internal.runners.model.MultipleFailureException;
import org.junit.runner.JUnitCore;
import org.junit.runner.Request;
import org.junit.runner.Result;

/**
 * @author jeff.collins
 */
@SuppressWarnings("deprecation")
public class UnfinalizingTestRunnerTest {	
	@Test
	public void testAnnotation() throws Exception {
		JUnitCore core = new JUnitCore();
		Result result = core.run(Request.method(TestClass.class, "testThis"));
		
		if (!result.wasSuccessful()) {
			if (result.getFailures().get(0).getException().getCause() instanceof MultipleFailureException) {
				Assert.fail(((MultipleFailureException)result.getFailures().get(0).getException().getCause()).getFailures().toString());
			} else {
				Assert.fail(result.getFailures().toString());
			}
		}
		
		Assert.assertTrue(result.getFailures().toString(), result.wasSuccessful());
		
		ClassLoader loader = UnfinalizingTestRunner.getLastCreatedClassLoader();
		
		// now check to see that the classes are not final
		Class<?> finalClass = loader.loadClass(FinalClass.class.getName());
		Assert.assertFalse("Class should have been non-final", Modifier.isFinal(finalClass.getModifiers()));
		Class<?> finalClass2 = loader.loadClass(FinalClass2.class.getName());
		Assert.assertFalse("Class 2 should have been non-final", Modifier.isFinal(finalClass2.getModifiers()));
	}
	
	@Test
	public void testMockFinal() {
		IMocksControl control = EasyMock.createStrictControl();
		try {
			control.createMock(TestClass.FinalClass.class);
			Assert.fail("FinalClass wasn't actually final or EasyMock has changed to support this case");
		} catch(IllegalArgumentException e) {
			
		}
		
		JUnitCore core = new JUnitCore();
		Result result = core.run(Request.method(TestClass.class, "testMockingFinal"));
		
		if (!result.wasSuccessful()) {
			if (result.getFailures().get(0).getException().getCause() instanceof MultipleFailureException) {
				Assert.fail(((MultipleFailureException)result.getFailures().get(0).getException().getCause()).getFailures().toString());
			} else {
				Assert.fail(result.getFailures().toString());
			}
		}
		
	}

	@Test
	public void testMockParentFinalMethod() {
		IMocksControl control = EasyMock.createStrictControl();
		try {
			control.createMock(TestClass.FinalChild.class);
			Assert.fail("FinalClass wasn't actually final or EasyMock has changed to support this case");
		} catch(IllegalArgumentException e) {
			
		}
		
		JUnitCore core = new JUnitCore();
		Result result = core.run(Request.method(TestClass.class, "testMockingParentFinal"));
		
		if (!result.wasSuccessful()) {
			if (result.getFailures().get(0).getException().getCause() instanceof MultipleFailureException) {
				Assert.fail(((MultipleFailureException)result.getFailures().get(0).getException().getCause()).getFailures().toString());
			} else {
				Assert.fail(result.getFailures().toString());
			}
		}
	}

	@Test
	public void testPrivateMethodAccess() throws Exception{
		try {
			TestClass.FinalClass.class.getDeclaredMethod("hiddenMethod").invoke((new TestClass()).new FinalClass());
			Assert.fail("FinalClass.hiddenMethod doesn't seem to be private");
		} catch(IllegalAccessException e) {
			
		}

		JUnitCore core = new JUnitCore();
		Result result = core.run(Request.method(TestClass.class, "testAllMethodsPublic"));
		
		if (!result.wasSuccessful()) {
			if (result.getFailures().get(0).getException().getCause() instanceof MultipleFailureException) {
				Assert.fail(((MultipleFailureException)result.getFailures().get(0).getException().getCause()).getFailures().toString());
			} else {
				Assert.fail(result.getFailures().toString());
			}
		}
	}
	
	@Test
	public void testPrivateClassAccess() throws Exception{
		Parent parent = Parent.getAHiddenImplementationOfParent();
		Assert.assertEquals(parent.getClass().getSimpleName(), "PrivateParentImpl");
		Assert.assertTrue("PrivateParentImpl should be private to start with", Modifier.isPrivate(parent.getClass().getModifiers()));		

		JUnitCore core = new JUnitCore();
		Result result = core.run(Request.method(TestClassForNonVisibleClasses.class, "testProxyingNonVisibleClass"));
		
		if (!result.wasSuccessful()) {
			if (result.getFailures().get(0).getException().getCause() instanceof MultipleFailureException) {
				Assert.fail(((MultipleFailureException)result.getFailures().get(0).getException().getCause()).getFailures().toString());
			} else {
				Assert.fail(result.getFailures().toString());
			}
		}
	}
}
